package info.textgrid.middleware;

import java.text.ParseException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import org.apache.commons.logging.Log;
import javax.xml.datatype.DatatypeConfigurationException;
import org.apache.commons.logging.LogFactory;
import org.elasticsearch.action.get.GetResponse;
import org.elasticsearch.search.SearchHit;

/**
 * <p>
 * The Dublin Core Field Loader takes the results of the ElasticSearch request and puts them into a
 * String-List. For each element of Dublin Core exists a separate list.
 * </p>
 * 
 * <p>
 * For each element two functions are necessary because the datatype "SeachHit" could contain more
 * then one results and the GetResponse just one.
 * </p>
 * 
 * @author Maximilian Brodhun, SUB Göttingen
 * @author Stefan E. Funk, SUB Göttingen
 * @version 2019-10-29
 * @since 2014-02-18
 */
public class DublinCoreFieldLoader {

  // **
  // STATICS
  // **

  private static org.apache.commons.logging.Log log = LogFactory.getLog(OAIPMHImpl.class);

  /**
   * This function takes the results of the ElasticSearch resuest 
   * and fills the String-Lists with the specific values for the dublin core field
   * @param responseOfGetRequest result of the ElasticSearch request from a getRequest (Single Result)
   * @param fields String List for the specific dublin core field
   * @return String List containing the values for the specific dublin core field
   */

  /**
   * @param responseWorkValues contains results of the ElasticSearch request
   * @param fields containing all fields from ElasticSearch related to this Dublin Core field
   * @return all elements mapped from TetxGrid metadata scheme to Dublin Core
   */
  public static List<String> setDate(GetResponse responseWorkValues, String[] fields) {

    // TODO We don't need this here, I suppose? We have to check the dates here!
    // return fillList(responseWorkValues, fields);

    List<String> dates = new ArrayList<String>();

    // Transform to OAI-PMH certificated time stamp
    if (responseWorkValues.isExists()) {
      for (String field : fields) {
        try {
          // FIXME DO SET VALUES VIA CONFIG FILE!
          if (responseWorkValues.getField(field) != null) {
            dates.add(OAIPMHUtilities
                .convertDateFormat(responseWorkValues.getField(field).getValue().toString())
                .toXMLFormat());
          }
        } catch (ParseException e) {
          // FIXME DO NOT USE INVALID DATE HERE!
          dates.add(responseWorkValues.getField(field).getValue().toString());
          log.debug(e);
          log.debug("Tried to parse date value: "
              + responseWorkValues.getField(field).getValue().toString());
        } catch (DatatypeConfigurationException e) {
          // FIXME DO NOT USE INVALID DATE HERE!
          dates.add(responseWorkValues.getField(field).getValue().toString());
          log.debug("Tried to configure date value: "
              + responseWorkValues.getField(field).getValue().toString());
          log.debug(e);
        }
      }
    }

    return dates;
  }

  /**
   * @param hit contains results of the ElasticSearch request
   * @param fields containing all fields from ElasticSearch related to this Dublin Core field
   * @return all elements mapped from TetxGrid metadata scheme to Dublin Core
   */
  public static List<String> setDate(SearchHit hit, String[] fields) {

    List<String> dates = new ArrayList<String>();

    // FIXME TADAA?
    System.out.println("TADA");

    for (String field : fields) {
      if (hit.getFields().get(field) != null) {
        try {
          dates.add(OAIPMHUtilities
              .convertDateFormat(hit.getFields().get(field).getValues().get(0).toString())
              .toXMLFormat());
        } catch (ParseException e) {
          // TODO Auto-generated catch block
          e.printStackTrace();
        } catch (DatatypeConfigurationException e) {
          // TODO Auto-generated catch block
          e.printStackTrace();
        }
      }
    }

    return dates;
  }


  public static List<String> fillList(GetResponse responseWorkValues, String[] fields) {

    List<String> list = new ArrayList<String>();

    if (responseWorkValues.isExists()) {
      if (fields != null) {
        for (String field : fields) {
          if (responseWorkValues.getSourceAsMap().get(field) == null) {
            String[] requestedField = field.split("\\.");
            Map<String, Object> nestedMap = responseWorkValues.getSourceAsMap();
            String valueOfRequestedField = null;
            Map<String, Object> nestedMap2 = null;
            for (int i = 0; i < requestedField.length; i++) {
              if (i < requestedField.length - 1 && nestedMap != null
                  && nestedMap.get(requestedField[i]) != null) {
                nestedMap2 = (Map<String, Object>) nestedMap.get(requestedField[i]);
              }
              if (i == requestedField.length - 1 &&
                  nestedMap2 != null && nestedMap2.get(requestedField[i]) != null) {
                valueOfRequestedField = nestedMap2.get(requestedField[i]).toString();
                if (valueOfRequestedField.length() > 0) {
                  list.add(valueOfRequestedField);
                }
              }
              nestedMap = nestedMap2;
            }
          } else {
            list.add(responseWorkValues.getSourceAsMap().get(field).toString());
          }
        }
      }
    }

    return list;
  }

  /**
   * @param hit
   * @param fields
   * @return
   */
  public static List<String> fillList(SearchHit hit, String[] fields) {

    List<String> list = new ArrayList<String>();

    if (fields != null) {
      for (String field : fields) {
        if (hit.getSourceAsMap().get(field) == null) {
          String[] requestedField = field.split("\\.");
          Map<String, Object> nestedMap = hit.getSourceAsMap();
          String valueOfRequestedField = null;
          Map<String, Object> nestedMap2 = null;
          for (int i = 0; i < requestedField.length; i++) {
            if (i < requestedField.length - 1 && nestedMap != null
                && nestedMap.get(requestedField[i]) != null) {
              try {
                nestedMap2 = (Map<String, Object>) nestedMap.get(requestedField[i]);
              } catch (ClassCastException cce) {

              }
            }
            if (i == requestedField.length - 1 &&
                nestedMap2 != null && nestedMap2.get(requestedField[i]) != null) {
              valueOfRequestedField = nestedMap2.get(requestedField[i]).toString();
              if (valueOfRequestedField.length() > 0) {
                list.add(valueOfRequestedField);
              }
            }
            nestedMap = nestedMap2;
          }
        } else {
          list.add(hit.getSourceAsMap().get(field).toString());
        }
      }
    }

    return list;
  }

}
